class Value:
    SWAGGER_VERSION = '2.0'
    PATH = 'path'
    QUERY = 'query'
    OBJECT = 'object'
    BODY = 'body'
    ARRAY = 'array'
    DEFAULT_RESPONSE = 'successful operation'
    INTEGER = 'integer'
    STRING = 'string'
    NUMBER = 'number'
    HEADER = 'header'
    TOKEN_DESCRIPTION = '<token>'
