from python_helper import Constant as c

REQUEST = 'Request'
RESPONSE = 'Response'

CONTROLLER_SPACE = f'[CONTROLLER]{c.SPACE}'
CONTROLLER_REQUEST = f'{CONTROLLER_SPACE}{REQUEST}'
CONTROLLER_RESPONSE = f'{CONTROLLER_SPACE}{RESPONSE}'

CLIENT_SPACE = f'[CLIENT]{c.SPACE}'
CLIENT_REQUEST = f'{CLIENT_SPACE}{REQUEST}'
CLIENT_RESPONSE = f'{CLIENT_SPACE}{RESPONSE}'

LISTENER_SPACE = f'[LISTENER]{c.SPACE}'
LISTENER_REQUEST = f'{LISTENER_SPACE}{REQUEST}'
LISTENER_RESPONSE = f'{LISTENER_SPACE}{RESPONSE}'

EMITTER_SPACE = f'[EMITTER]{c.SPACE}'
EMITTER_REQUEST = f'{EMITTER_SPACE}{REQUEST}'
EMITTER_RESPONSE = f'{EMITTER_SPACE}{RESPONSE}'
